#/* Copyright 1998-2011 Research Foundation State University of New York */
#
#/* This file is part of QuB.                                            */
#
#/* QuB is free software; you can redistribute it and/or modify          */
#/* it under the terms of the GNU General Public License as published by */
#/* the Free Software Foundation, either version 3 of the License, or    */
#/* (at your option) any later version.                                  */
#
#/* QuB is distributed in the hope that it will be useful,               */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
#/* GNU General Public License for more details.                         */
#
#/* You should have received a copy of the GNU General Public License,   */
#/* named LICENSE.txt, in the QuB program directory.  If not, see        */
#/* <http://www.gnu.org/licenses/>.                                      */

import QUB
import Commands



#def CountRates(mdl):
#    nrate = 0
#    rate = mdl.find('Rates').find('Rate')
#    while rate:
#        nrate = nrate + 1
#        rate = rate.nextSameName()
#    return nrate


#def ThreeRateFilter(mdl):
#    if CountRates(mdl) > 3:
#        return
#    return mdl

#Commands.Register("ModelFilters", "Max 3 Rates", ThreeRateFilter)

def lsub(lst1, lst2):
    lst3 = lst1[:]
    for x in lst2:
        if x in lst3:
            lst3.remove(x)
    return lst3

class Incidents:
    def __init__(self, mdl):
        self.mdl = mdl
        self.states = []
        self.rates = []
        self.connectionsFromState = []

        state = mdl.find('States').find('State')
        while state:
            self.states.append(state)
            self.connectionsFromState.append([])
            state = state.nextSameName()
        rate = mdl.find('Rates').find('Rate')
        while rate:
            self.rates.append(rate)
            self.connectionsFromState[ rate['States'].data[0] ].append(rate['States'].data[1])
            self.connectionsFromState[ rate['States'].data[1] ].append(rate['States'].data[0])
            rate = rate.nextSameName()            

def FilterCligCOO(mdl):
    incidents = Incidents(mdl)
    def isOpen(state_ix):
        return incidents.states[state_ix]['Class'].data[0] == 1
    def hasOneOpenConnection(state_ix):
        return ((len(incidents.connectionsFromState[state_ix]) == 1)
            and isOpen(incidents.connectionsFromState[state_ix][0]))
    starters = [ix for ix in range(len(incidents.states)) if (isOpen(ix) and hasOneOpenConnection(ix))]
    if not len(starters):
        return

    def extendPath(path, rq_len):
        #if len(path) >= rq_len:
        #    return path
        options = lsub(incidents.connectionsFromState[path[len(path)-1]], path)
        if not len(options):
            return path
        for opt in options:
            trypath = extendPath(path+[opt], rq_len)
            if len(trypath) == rq_len:
                return trypath
        return trypath

    def buildpath(starter, rq_len):
        path = [starter, incidents.connectionsFromState[starter][0]]
        return extendPath(path, rq_len)

    def buildpaths(starters, rq_len):
        for starter in starters:
            trypath = buildpath(starter, rq_len)
            # QUB.Report(('tried %d: ' % starter) + str(trypath))
            if len(trypath) >= rq_len:
                return trypath
        return []

    path = buildpaths(starters, 4)
    if len(path) >= 4:
        c0 = path[len(path)-1]
        c1 = path[len(path)-2]
        for rate in incidents.rates:
            states = rate['States'].data
            if states[0] == c0 and states[1] == c1:
                rate['P'].data[0] = 1
            elif states[1] == c0 and states[0] == c1:
                rate['P'].data[1] = 1
        return mdl
    else:
        return


# has a core of C* - C - C - O - O
# final O is incident only on other O
# make first C - C conc.dependent going forward

Commands.Register('ModelFilters', 'CligCOO', FilterCligCOO)
